from typing import Literal

from .base import (
    BasePlayer,
    BaseChatEvent,
    BaseJoinEvent,
    BaseQuitEvent
)


class Player(BasePlayer):
    """原版 玩家信息"""


class MinecraftPlayerChatEvent(BaseChatEvent):
    """原版 玩家聊天事件"""
    event_name: Literal["MinecraftPlayerChatEvent"]
    player: Player


class MinecraftPlayerJoinEvent(BaseJoinEvent):
    """原版 玩家加入事件"""
    event_name: Literal["MinecraftPlayerJoinEvent"]
    player: Player


class MinecraftPlayerQuitEvent(BaseQuitEvent):
    """原版 玩家退出事件"""
    event_name: Literal["MinecraftPlayerQuitEvent"]
    player: Player



from typing import Optional, Literal
from .base import (
    BasePlayer,
    BaseChatEvent,
    BaseDeathEvent,
    BaseJoinEvent,
    BaseQuitEvent,
    BasePlayerCommandEvent,
)


class Player(BasePlayer):
    """玩家信息"""
    uuid: Optional[str] = None
    display_name: Optional[str] = None
    player_list_name: Optional[str] = None
    is_health_scaled: Optional[bool] = None
    address: Optional[str] = None
    is_sprinting: Optional[bool] = None
    walk_speed: Optional[float] = None
    fly_speed: Optional[float] = None
    is_sneaking: Optional[bool] = None
    level: Optional[int] = None
    is_flying: Optional[bool] = None
    ping: Optional[int] = None
    """Spigot API 1.12.2 Player 无 ping 属性"""
    allow_flight: Optional[bool] = None
    locale: Optional[str] = None
    health_scale: Optional[float] = None
    player_time_offset: Optional[int] = None
    exp: Optional[float] = None
    total_exp: Optional[int] = None
    player_time: Optional[int] = None
    is_player_time_relative: Optional[bool] = None
    is_op: Optional[bool] = None


class AsyncPlayerChatEvent(BaseChatEvent):
    """Spigot API AsyncPlayerChatEvent"""
    event_name: Literal["AsyncPlayerChatEvent"]
    player: Player


class PlayerCommandPreprocessEvent(BasePlayerCommandEvent):
    """Spigot API PlayerCommandPreprocessEvent"""
    event_name: Literal["PlayerCommandPreprocessEvent"]
    player: Player


class PlayerJoinEvent(BaseJoinEvent):
    """Spigot API PlayerJoinEvent"""
    event_name: Literal["PlayerJoinEvent"]
    player: Player


class PlayerQuitEvent(BaseQuitEvent):
    """Spigot API PlayerQuitEvent"""
    event_name: Literal["PlayerQuitEvent"]
    player: Player


class PlayerDeathEvent(BaseDeathEvent):
    """Spigot API PlayerDeathEvent"""
    event_name: Literal["PlayerDeathEvent"]
    player: Player



from typing import Literal

from pydantic import BaseModel


class BaseEvent(BaseModel):
    """事件基类"""
    post_type: str
    event_name: str
    server_name: str
    sub_type: str


class BasePlayer(BaseModel):
    """玩家信息基类"""
    nickname: str


class BaseMessageEvent(BaseEvent):
    """消息事件基类"""
    post_type: Literal["message"]
    player: BasePlayer
    message: str


class BaseChatEvent(BaseMessageEvent):
    """玩家聊天事件基类"""
    sub_type: Literal["chat"]


class BasePlayerCommandEvent(BaseMessageEvent):
    """玩家执行命令事件基类"""
    sub_type: Literal["player_command"]


class BaseDeathEvent(BaseMessageEvent):
    """玩家死亡事件基类"""
    sub_type: Literal["death"]


class BaseNoticeEvent(BaseEvent):
    """通知事件基类"""
    post_type: Literal["notice"]
    player: BasePlayer


class BaseJoinEvent(BaseNoticeEvent):
    """玩家加入事件基类"""
    sub_type: Literal["join"]


class BaseQuitEvent(BaseNoticeEvent):
    """玩家退出事件基类"""
    sub_type: Literal["quit"]



"""
发送共用
"""
import contextlib
from typing import Union

from aiomcrcon import ClientNotConnectedError
from nonebot import logger

from ...config import plugin_config, Client, CLIENTS


async def remove_client(server_name: str):
    """
    移除客户端
    :param server_name: 服务器名
    """
    if client := CLIENTS.get(server_name):
        if client.websocket:
            with contextlib.suppress(Exception):
                await client.websocket.close()
        if client.rcon:
            await client.rcon.close()
        del CLIENTS[server_name]
        logger.info(f"[MC_QQ]丨[Server:{server_name}] 的 WebSocket 客户端已移除")


async def send_common_cmd_to_mc(client: Client, cmd: str) -> Union[str, None]:
    """
    准备发送通用命令到 MC
    :param client: 客户端
    :param cmd: 命令
    """
    server = plugin_config.mc_qq_server_dict.get(client.server_name)
    # 先判断是否有Rcon进行发送
    if client.rcon and server.rcon_cmd:
        return await send_common_cmd_to_mc_by_rcon(client=client, cmd=cmd)
    elif client.websocket:
        # await send_common_to_mc_by_ws(client=client, message=cmd)
        return "WebSocket 暂不支持发送命令"
    else:
        await remove_client(client.server_name)
    return None


async def send_common_cmd_to_mc_by_rcon(client: Client, cmd: str) -> Union[str, None]:
    """
    准备发送通用 Rcon 命令到 MC
    :param client:
    :param cmd:
    :return: 返回信息
    """
    try:
        back_msg = await client.rcon.send_cmd(cmd)
        logger.success(f"[MC_QQ_Rcon]丨发送至 [server:{client.server_name}] 的消息 \"{cmd}\"")
        return back_msg[0]
    except ClientNotConnectedError as e:
        logger.error(f"[MC_QQ_Rcon]丨发送至 [Server:{client.server_name}] 的过程中出现了错误：{e}")
        await remove_client(client.server_name)
    return None


async def send_common_to_mc_by_ws(client: Client, message: str) -> None:
    """
    准备发送通用 WebSocket 命令到 MC
    :param client:
    :param message:
    :return: 返回信息
    """
    await client.websocket.send_text(message)
    return None



"""
格式化MC消息
"""

import json

from nonebot import logger
from nonebot.adapters.qq import ActionFailed

from ..send import (
    send_msg_to_onebot_group,
    send_msg_to_onebot_guild,
    send_msg_to_qq_guild
)
from ...config import Server, plugin_config
from ...event import (
    event_dict,
    BaseEvent,
    BaseChatEvent,
    BaseDeathEvent,
    BaseJoinEvent,
    BaseQuitEvent,
    BasePlayerCommandEvent
)


def _msg_to_qq_process(event: BaseEvent) -> str:
    """处理来自MC的消息，并返回处理后的消息"""
    if isinstance(event, BaseChatEvent):
        return f"{event.player.nickname} 说：{event.message}"
    elif isinstance(event, BaseDeathEvent):
        return f"{event.player.nickname} {event.message}"
    elif isinstance(event, BaseJoinEvent):
        return f"{event.player.nickname} 加入了游戏"
    elif isinstance(event, BaseQuitEvent):
        return f"{event.player.nickname} 离开了游戏"
    elif isinstance(event, BasePlayerCommandEvent):
        # return f"{event.player.nickname} 执行了命令：{event.message}"
        # logger.info(f"[MC_QQ]丨from [{event.server_name}] {event.player.nickname} 执行了命令：{event.message}")
        return ""
    else:
        return ""


async def send_msg_from_mc_common(message: str):
    """处理来自MC的消息，并准备发送"""
    json_msg = json.loads(message)
    event = event_dict[json_msg["event_name"]].parse_obj(json_msg)

    if not (msg := _msg_to_qq_process(event_dict[json_msg["event_name"]].parse_obj(json_msg))):
        return

    if plugin_config.mc_qq_display_server_name:
        msg = f"[{event.server_name}] {msg}"

    # 从服务器字典中获取服务器对应的群聊信息并发送
    if per_server := plugin_config.mc_qq_server_dict.get(event.server_name):
        per_server: Server  # 标记类型
        for per_group in per_server.group_list:
            # 判断发送的目标群聊是否为 OneBot适配器
            if per_group.adapter == "onebot":
                await send_msg_to_onebot_group(
                    bot_id=str(per_group.bot_id),
                    group_id=per_group.group_id,
                    server_name=event.server_name,
                    message=msg
                )
            # TODO 判断发送的目标群聊是否为 QQ适配器，群私域 Bot 暂未开放
            # elif per_group.adapter == "qq":
            #     await bot.send_private_msg(
            #         user_id=per_group,
            #         message=msg
            #     )
            #     logger.info(f"[MC_QQ]丨from [{event.server_name}] to [群:{per_group}] \"{msg}\"")
        # 判断是否发送到频道
        for per_guild in per_server.guild_list:
            # 判断发送的目标频道是否为 OneBot适配器
            if per_guild.adapter == "onebot":
                await send_msg_to_onebot_guild(
                    server_name=event.server_name,
                    bot_id=str(per_guild.bot_id),
                    guild_id=per_guild.guild_id,
                    channel_id=per_guild.channel_id,
                    message=msg
                )
            elif per_guild.adapter == "qq":
                try:
                    await send_msg_to_qq_guild(
                        server_name=event.server_name,
                        bot_id=str(per_guild.bot_id),
                        channel_id=str(per_guild.channel_id),
                        message=msg
                    )
                except ActionFailed as e:
                    logger.debug(f"[MC_QQ]丨发送到频道的消息在审核中")



from nonebot.adapters.onebot.v11 import Bot as OneBot
from nonebot.adapters.qq import MessageCreateEvent
from nonebot.internal.permission import Permission
from nonebot_plugin_guild_patch import GuildMessageEvent

from .config import plugin_config


async def _onebot_guild_admin(
        bot: OneBot,
        event: GuildMessageEvent,
):
    """检测是否为 OneBot适配器 频道管理员"""
    roles = set(
        role["role_name"]
        for role in (
            await bot.get_guild_member_profile(
                guild_id=event.guild_id, user_id=event.user_id
            )
        )["roles"]
    )
    return bool(roles & set(plugin_config.mc_qq_guild_admin_roles))


async def _qq_guild_admin(
        event: MessageCreateEvent,
):
    """检测是否为 QQ适配器 频道管理员"""
    return bool(set(event.member.roles) & set(plugin_config.mc_qq_guild_admin_roles))


ONEBOT_GUILD_ADMIN: Permission = Permission(_onebot_guild_admin)
"""OneBot适配器 频道管理员权限"""
QQ_GUILD_ADMIN: Permission = Permission(_qq_guild_admin)
"""QQ适配器 频道管理员权限"""



"""
返回体
"""

from __future__ import annotations

import json
from enum import Enum
from typing import List, Optional

from pydantic import BaseModel


class ActionEvent(BaseModel):
    """
    点击事件
    """
    click_event_url: Optional[str] = None
    hover_event_text: Optional[str] = None


class TextColor(Enum):
    """
    颜色枚举
    """
    BLACK = "black"
    DARK_BLUE = "dark_blue"
    DARK_GREEN = "dark_green"
    DARK_AQUA = "dark_aqua"
    DARK_RED = "dark_red"
    DARK_PURPLE = "dark_purple"
    GOLD = "gold"
    GRAY = "gray"
    DARK_GRAY = "dark_gray"
    BLUE = "blue"
    GREEN = "green"
    AQUA = "aqua"
    RED = "red"
    LIGHT_PURPLE = "light_purple"
    YELLOW = "yellow"
    WHITE = "white"


class MessageItem(BaseModel):
    """
    消息体
    """
    msg_text: Optional[str] = None
    color: Optional[TextColor] = None
    action_event: Optional[ActionEvent] = None


class WebSocketSendBody(BaseModel):
    """
    websocket 发送消息的body
    """
    message: List[MessageItem] = []


class ChatImageModel(BaseModel):
    """
    ChatImage Mod 图片
    """
    url: Optional[str] = None
    name: Optional[str] = None

    def __str__(self):
        return f"[[CICode,url={self.url}]]"


"""
Rcon
"""


class RconClickEventEnum(Enum):
    """
    点击事件枚举
    """
    OPEN_URL = "open_url"
    OPEN_FILE = "open_file"
    RUN_COMMAND = "run_command"
    SUGGEST_COMMAND = "suggest_command"
    CHANGE_PAGE = "change_page"  # 仅用于书翻页
    COPY_TO_CLIPBOARD = "copy_to_clipboard"


class RconHoverEventEnum(Enum):
    """
    悬停事件枚举
    """
    SHOW_TEXT = "show_text"
    SHOW_ITEM = "show_item"
    SHOW_ENTITY = "show_entity"


class RconFontEnum(Enum):
    """
    字体枚举
    """
    DEFAULT = "minecraft:default"


class RconClickEvent(BaseModel):
    """
    点击事件
    """
    action: Optional[RconClickEventEnum] = None
    value: Optional[str] = None


class RconBaseComponent(BaseModel):
    """
    基本组件
    """
    text: Optional[str] = None
    color: Optional[TextColor] = None
    bold: Optional[bool] = None
    italic: Optional[bool] = None
    underlined: Optional[bool] = None
    strikethrough: Optional[bool] = None
    obfuscated: Optional[bool] = None
    insertion: Optional[str] = None
    font: Optional[RconFontEnum] = None
    score: Optional[dict] = None
    selector: Optional[str] = None
    block: Optional[str] = None
    translate: Optional[str] = None


class RconHoverEvent(BaseModel):
    """
    悬停事件
    """
    action: Optional[RconHoverEventEnum] = None
    contents: Optional[List[RconBaseComponent]] = None


class RconTextComponent(RconBaseComponent):
    """
    文本组件
    """
    click_event: Optional[RconClickEvent] = None
    hover_event: Optional[RconHoverEvent] = None


class RconSendBody(BaseModel):
    """
    rcon 发送消息的body
    """
    message: List[RconTextComponent] = []

    def get_tellraw(self) -> str:
        """
        获取tellraw命令
        :return:
        """
        # 将body中包含None的字段删除
        body_list = json.loads(self.json()).get("message")
        for i in body_list:
            for k, v in list(i.items()):
                if v is None:
                    del i[k]
        # 在首位置添加一个空字符串
        return f"tellraw @a {json.dumps(body_list)}"



"""
配置文件
"""

from aiomcrcon import Client as RconClient
from nonebot import get_driver
from nonebot.drivers.websockets import WebSocket
from typing import Optional, List, Dict
from pydantic import BaseModel, Extra, Field


class Client:
    """MC_QQ 客户端"""
    server_name: str
    websocket: WebSocket
    rcon: Optional[RconClient] = None

    def __init__(
            self, server_name: str,
            websocket: WebSocket,
            rcon: Optional[RconClient] = None
    ):
        self.server_name: str = server_name
        self.websocket: WebSocket = websocket
        self.rcon: Optional[RconClient] = rcon


CLIENTS: Dict[str, Client] = {}


class Guild(BaseModel):
    """频道配置"""
    # 频道ID，QQ适配器不需要频道ID
    guild_id: Optional[int] = None
    # 子频道ID
    channel_id: Optional[int] = None
    # 适配器类型
    adapter: Optional[str] = None
    # Bot ID 优先使用所选Bot发送消息
    bot_id: Optional[int] = None


class Group(BaseModel):
    """群配置"""
    # 群ID
    group_id: int
    # 适配器类型
    adapter: Optional[str] = None
    # Bot ID
    bot_id: Optional[int] = None


class Server(BaseModel):
    """服务器配置"""
    # 服务器群列表
    group_list: Optional[List[Group]] = []
    # 服务器频道列表
    guild_list: Optional[List[Guild]] = []
    # 是否开启 Rcon 消息
    rcon_msg: Optional[bool] = False
    # 是否开启 Rcon 命令
    rcon_cmd: Optional[bool] = False
    # Rcon 密码
    rcon_password: Optional[str] = "password"
    # Rcon 端口
    rcon_port: Optional[int] = 25575


class Config(BaseModel, extra=Extra.ignore):
    """配置"""
    # 路由地址
    mc_qq_ws_url: Optional[str] = "/mcqq"
    # 是否发送群聊名称
    mc_qq_send_group_name: Optional[bool] = False
    # 是否显示服务器名称
    mc_qq_display_server_name: Optional[bool] = False
    # 服务器列表字典
    mc_qq_server_dict: Optional[Dict[str, Server]] = Field(default_factory=dict)
    # MC_QQ 频道管理员身份组
    mc_qq_guild_admin_roles: Optional[List[str]] = ["频道主", "超级管理员"]
    # MC_QQ 启用 ChatImage MOD
    mc_qq_chat_image_enable: Optional[bool] = False
    # MC_QQ Rcon 启用 ClickAction
    mc_qq_rcon_click_action_enable: Optional[bool] = False
    # MC_QQ Rcon 启用 HoverEvent
    mc_qq_rcon_hover_event_enable: Optional[bool] = False


plugin_config: Config = Config.parse_obj(get_driver().config)



"""
工具包
"""
from typing import Union

from aiomcrcon import (
    Client as RconClient,
    RCONConnectionError,
    IncorrectPasswordError
)
from nonebot import logger
from nonebot.adapters.onebot.v11 import Bot as OneBot, GroupMessageEvent
from nonebot.adapters.qq import Bot as QQBot, MessageCreateEvent
from nonebot_plugin_guild_patch import GuildMessageEvent

from .parse import (
    parse_qq_msg_to_basemodel,
    parse_onebot_rcon_msg_to_basemodel,
    parse_onebot_msg_to_basemodel,
    parse_qq_rcon_msg_to_basemodel
)
from .send import (
    send_common_to_mc_by_ws,
    send_common_cmd_to_mc,
    send_common_cmd_to_mc_by_rcon
)
from ..config import Client, CLIENTS, plugin_config
from ..model.return_body import *


async def send_msg_to_mc(
        bot: Union[OneBot, QQBot],
        event: Union[GroupMessageEvent, GuildMessageEvent, MessageCreateEvent],
):
    """
    发送消息到 MC 服务器
    :param bot:
    :param event: 事件
    """
    if client_list := await get_clients(event=event):
        for client in client_list:
            if client:
                server = plugin_config.mc_qq_server_dict.get(client.server_name)

                # 先判断是否有Rcon进行发送
                if client.rcon and server.rcon_msg:
                    if isinstance(event, MessageCreateEvent):
                        message = await parse_qq_rcon_msg_to_basemodel(event=event, bot=bot)
                        if len(message.get_tellraw()) > 256:
                            return "消息过长，无法发送"
                    else:
                        message = await parse_onebot_rcon_msg_to_basemodel(event=event, bot=bot)
                        if len(message.get_tellraw()) > 256:
                            return "消息过长，无法发送"
                    await send_common_cmd_to_mc_by_rcon(client=client, cmd=message.get_tellraw())
                else:
                    if isinstance(event, MessageCreateEvent):
                        message = (await parse_qq_msg_to_basemodel(event=event, bot=bot)).json(ensure_ascii=False)
                    else:
                        message = (await parse_onebot_msg_to_basemodel(event=event, bot=bot)).json(ensure_ascii=False)
                    await send_common_to_mc_by_ws(client=client, message=message)
    return None


async def send_cmd_to_mc(
        event: Union[GroupMessageEvent, GuildMessageEvent, MessageCreateEvent],
        cmd: str
) -> str:
    """
    发送命令到 MC 服务器
    :param event: 事件
    :param cmd: 命令
    :return: 返回信息
    """
    back_msg = ""
    if client_list := await get_clients(event=event):
        if len(client_list) == 1:
            back_msg = await send_common_cmd_to_mc(client=client_list[0], cmd=cmd)
        else:
            for client in client_list:
                if back_info := await send_common_cmd_to_mc_by_rcon(client=client, cmd=cmd):
                    back_msg += f"{client.server_name} {back_info}"
    return back_msg


async def get_clients(event: Union[GroupMessageEvent, GuildMessageEvent, MessageCreateEvent]) -> List[Client]:
    """
    获取 服务器名、ws客户端, 返回client列表
    :param event: 事件
    :return: client列表
    """
    res: List[Client] = []
    for per_server_name, per_server in plugin_config.mc_qq_server_dict.items():
        if isinstance(event, GroupMessageEvent):
            for per_group in per_server.group_list:
                if per_group.group_id == event.group_id:
                    res.append(CLIENTS.get(per_server_name))
        elif isinstance(event, GuildMessageEvent | MessageCreateEvent):
            for per_guild in per_server.guild_list:
                if str(per_guild.guild_id) == str(event.guild_id) and str(per_guild.channel_id) == str(
                        event.channel_id):
                    res.append(CLIENTS.get(per_server_name))
    return res


async def rcon_connect(rcon_client: RconClient, server_name: str):
    """
    连接 Rcon
    :param rcon_client: Rcon 客户端
    :param server_name: 服务器名
    """
    try:
        await rcon_client.connect()
        logger.success(f"[MC_QQ]丨[Server:{server_name}] 的Rcon连接成功")
    except RCONConnectionError as e:
        logger.error(f"[MC_QQ]丨[Server:{server_name}] 的Rcon连接失败：{str(e)}")
    except IncorrectPasswordError as e:
        logger.error(f"[MC_QQ]丨[Server:{server_name}] 的Rcon密码错误：{str(e)}")



"""
发送消息
"""

from typing import Union

from nonebot import logger, get_bot
from nonebot.adapters.onebot.v11 import Bot as OneBot
from nonebot.adapters.qq import Bot as QQBot
from nonebot.exception import FinishedException


def _get_available_bot(bot_id: str, server_name: str) -> Union[OneBot, QQBot]:
    """
    获取可用 Bot
    :param bot_id: Bot ID
    :param server_name: 服务器名
    :return: Bot
    """
    try:
        bot = get_bot(bot_id)
    except KeyError as e:
        logger.warning(
            f"[MC_QQ]丨Server: {server_name} 的 Bot {bot_id} 不存在，将尝试使用其他Bot发送信息"
        )
        try:
            bot = get_bot()
        except ValueError as e:
            logger.warning(
                f"[MC_QQ]丨当前无其他Bot可用，将跳过发送 Server: {server_name} 的信息"
            )
            raise FinishedException
    except ValueError as e:
        logger.warning(
            f"[MC_QQ]丨当前无其他Bot可用，将跳过发送 Server: {server_name} 的信息"
        )
        raise FinishedException
    return bot


async def send_msg_to_onebot_group(bot_id: str, group_id: int, server_name: str, message: str):
    """
    发送消息到 OneBot 适配器群聊
    :param server_name: 服务器名
    :param bot_id: Bot ID
    :param group_id: 群号
    :param message: 消息
    :return:
    """
    bot: OneBot = _get_available_bot(bot_id=bot_id, server_name=server_name)
    await bot.send_group_msg(
        group_id=group_id,
        message=message
    )
    logger.info(f"[MC_QQ]丨from [{server_name}] to [群:{group_id}] \"{message}\"")


async def send_msg_to_onebot_guild(bot_id: str, guild_id: int, channel_id: int, server_name: str, message: str):
    """
    发送消息到 OneBot 适配器频道
    :param server_name:
    :param channel_id:
    :param bot_id:
    :param guild_id:
    :param message:
    :return:
    """
    bot: OneBot = _get_available_bot(bot_id=bot_id, server_name=server_name)
    await bot.send_guild_channel_msg(
        guild_id=guild_id,
        channel_id=channel_id,
        message=message
    )
    logger.info(
        f"[MC_QQ]丨from [{server_name}] to [频道:{guild_id}/{channel_id}] \"{message}\""
    )


async def send_msg_to_qq_guild(bot_id: str, channel_id: str, server_name: str, message: str):
    """
     发送消息到 QQ 适配器频道
    :param server_name: 服务器名
    :param bot_id: Bot ID
    :param channel_id: 频道号
    :param message: 消息
    :return:
    """
    bot = _get_available_bot(bot_id=bot_id, server_name=server_name)
    await bot.send_to_channel(
        channel_id=channel_id,
        message=message
    )
    logger.info(
        f"[MC_QQ]丨from [{server_name}] to [频道:{channel_id}] \"{message}\""
    )



"""
事件包
"""

from .base import (
    BaseEvent,
    BasePlayer,
    BaseMessageEvent,
    BaseChatEvent,
    BasePlayerCommandEvent,
    BaseNoticeEvent,
    BaseDeathEvent,
    BaseJoinEvent,
    BaseQuitEvent
)
from .forge import (
    Player as ForgePlayer,
    ForgePlayerLoggedInEvent,
    ForgePlayerLoggedOutEvent,
    ForgeServerChatEvent,
    ForgePlayerRespawnEvent
)
from .minecraft import (
    Player as MinecraftPlayer,
    MinecraftPlayerChatEvent,
    MinecraftPlayerJoinEvent,
    MinecraftPlayerQuitEvent
)
from .spigot import (
    Player as SpigotPlayer,
    AsyncPlayerChatEvent,
    PlayerDeathEvent,
    PlayerJoinEvent,
    PlayerQuitEvent,
    PlayerCommandPreprocessEvent
)

event_dict = {
    # 原版
    "MinecraftPlayerJoinEvent": MinecraftPlayerJoinEvent,
    "MinecraftPlayerQuitEvent": MinecraftPlayerQuitEvent,
    "MinecraftPlayerChatEvent": MinecraftPlayerChatEvent,
    # Spigot
    "AsyncPlayerChatEvent": AsyncPlayerChatEvent,
    "PlayerDeathEvent": PlayerDeathEvent,
    "PlayerJoinEvent": PlayerJoinEvent,
    "PlayerQuitEvent": PlayerQuitEvent,
    "PlayerCommandPreprocessEvent": PlayerCommandPreprocessEvent,
    # Forge
    "ForgeServerChatEvent": ForgeServerChatEvent,
    "ForgePlayerLoggedInEvent": ForgePlayerLoggedInEvent,
    "ForgePlayerLoggedOutEvent": ForgePlayerLoggedOutEvent,
    "ForgePlayerRespawnEvent": ForgePlayerRespawnEvent,
}



from typing import Optional, Literal

from .base import (
    BasePlayer,
    BaseChatEvent,
    BaseDeathEvent,
    BaseJoinEvent,
    BaseQuitEvent
)


class Player(BasePlayer):
    """玩家信息"""
    nickname: Optional[str] = None
    uuid: Optional[str] = None
    ipAddress: Optional[str] = None
    level: Optional[str] = None
    """地图？"""
    speed: Optional[float] = None


class ForgeServerChatEvent(BaseChatEvent):
    """Forge API ServerChatEvent"""
    event_name: Literal["ForgeServerChatEvent"]
    player: Player


class ForgePlayerLoggedInEvent(BaseJoinEvent):
    """Forge API PlayerLoggedInEvent"""
    event_name: Literal["ForgePlayerLoggedInEvent"]
    player: Player


class ForgePlayerLoggedOutEvent(BaseQuitEvent):
    """Forge API PlayerLoggedOutEvent"""
    event_name: Literal["ForgePlayerLoggedOutEvent"]
    player: Player


class ForgePlayerRespawnEvent(BaseDeathEvent):
    """Forge API ForgePlayerRespawnEvent"""
    event_name: Literal["ForgePlayerRespawnEvent"]
    player: Player



"""
格式化频道消息
"""

from typing import Union

from nonebot.adapters.onebot.v11 import Bot as OneBot, GroupMessageEvent
from nonebot.adapters.qq.models import Member, Guild, Channel
from nonebot_plugin_guild_patch import GuildMessageEvent
from nonebot.adapters.qq import Bot as QQBot, MessageCreateEvent

from ...model.return_body import *
from ...config import plugin_config


async def _get_onebot_member_nickname(
        bot: OneBot,
        event: Union[GroupMessageEvent, GuildMessageEvent],
        user_id: Union[int, str]
) -> str:
    """
    获取 OneBot 群/频道成员昵称
    :param bot: OneBot
    :param event: 事件
    :param user_id: 成员QQ号/频道ID
    :return: 成员昵称
    """
    # 判断从 群/频道 获取成员信息
    if isinstance(event, GroupMessageEvent):
        # 如果获取发送者的昵称
        if event.user_id == int(user_id):
            # 如果群名片为空，则发送昵称
            return event.sender.card or event.sender.nickname
        # 如果获取其他人的昵称
        else:
            return (await bot.get_group_member_info(
                group_id=event.group_id,
                user_id=user_id,
                no_cache=True
            ))['nickname']
    elif isinstance(event, GuildMessageEvent):
        # 返回频道成员昵称
        if event.user_id == user_id:
            return event.sender.nickname
        else:
            return (await bot.get_guild_member_profile(
                guild_id=event.guild_id,
                user_id=user_id))['nickname']


async def get_qq_member_nickname(
        bot: QQBot,
        event: MessageCreateEvent,
        user_id: str
) -> str:
    """
    获取 QQ 群/频道成员昵称
    :return: 成员昵称
    """
    # 判断事件发送者是否为本人
    if event.author.id == user_id:
        return event.member.nick or event.author.username
    else:
        member: Member = await bot.get_member(guild_id=event.guild_id, user_id=user_id)
        return member.nick or member.user.username


async def parse_onebot_msg_to_basemodel(
        bot: OneBot,
        event: Union[GroupMessageEvent, GuildMessageEvent]
) -> WebSocketSendBody:
    """
    :param bot: OneBot
    :param event: 事件
    :return: WebSocketSendBody
    """
    websocket_send_body = WebSocketSendBody()
    websocket_send_body.message = []

    # 发送群聊名称
    if plugin_config.mc_qq_send_group_name:
        group_name = MessageItem()
        group_name.color = TextColor.GREEN
        if isinstance(event, GroupMessageEvent):
            group_name.msg_text = (await bot.get_group_info(group_id=event.group_id))['group_name']
        elif isinstance(event, GuildMessageEvent):
            guild_name = (await bot.get_guild_meta_by_guest(guild_id=event.guild_id))['guild_name']
            for per_channel in (await bot.get_guild_channel_list(guild_id=event.guild_id, no_cache=True)):
                if str(event.channel_id) == per_channel['channel_id']:
                    channel_name = per_channel['channel_name']
                    group_name.msg_text = f"{guild_name}丨{channel_name}"
                    break
        websocket_send_body.message.append(group_name)

    member_nickname = await _get_onebot_member_nickname(bot, event, event.user_id)

    sender_name = MessageItem()
    sender_name.msg_text = member_nickname + " "
    sender_name.color = TextColor.WHITE
    websocket_send_body.message.append(sender_name)

    sender_say = MessageItem()
    sender_say.msg_text = "说："
    sender_say.color = TextColor.GOLD
    websocket_send_body.message.append(sender_say)

    for msg in event.message:
        per_msg = MessageItem()
        per_msg.action_event = ActionEvent()

        # 文本
        if msg.type == "text":
            per_msg.msg_text = msg.data['text'].replace("\r", "").replace("\n", "\n * ")
            per_msg.color = TextColor.WHITE
        # 图片
        elif msg.type == "image":
            if plugin_config.mc_qq_chat_image_enable:
                per_msg.msg_text = str(ChatImageModel(url=msg.data['url'], name="图片"))
            else:
                per_msg.msg_text = "[图片]"
                per_msg.action_event.click_event_url = msg.data['url']
                per_msg.action_event.hover_event_text = "[查看图片]"
            per_msg.color = TextColor.AQUA
        # 表情
        elif msg.type == "face":
            per_msg.msg_text = '[表情]'
            per_msg.color = TextColor.YELLOW
        # 语音
        elif msg.type == "record":
            per_msg.msg_text = '[语音]'
            per_msg.color = TextColor.GREEN
        # 视频
        elif msg.type == "video":
            per_msg.msg_text = '[视频]'
            per_msg.action_event.click_event_url = msg.data['url']
            per_msg.action_event.hover_event_text = "[查看视频]"
            per_msg.color = TextColor.LIGHT_PURPLE
        # @
        elif msg.type == "at":
            # 获取被@ 群/频道 昵称
            at_member_nickname = await _get_onebot_member_nickname(bot, event, msg.data['qq'])
            per_msg.msg_text = f"@{at_member_nickname} "
            per_msg.color = TextColor.GREEN
        # share
        elif msg.type == "share":
            per_msg.msg_text = "[分享]"
            per_msg.action_event.click_event_url = msg.data['url']
            per_msg.action_event.hover_event_text = "[查看分享]"
            per_msg.color = TextColor.YELLOW
        # forward
        elif msg.type == "forward":
            # TODO 将合并转发消息拼接为字符串
            # 获取合并转发 await bot.get_forward_msg(message_id=event.message_id)
            per_msg.msg_text = '[合并转发]'
        else:
            per_msg.msg_text = f"[{msg.type}]"
            per_msg.color = TextColor.WHITE
        websocket_send_body.message.append(per_msg)
    return websocket_send_body


async def parse_qq_msg_to_basemodel(
        bot: QQBot,
        event: MessageCreateEvent
) -> WebSocketSendBody:
    """
    格式化 QQ频道 消息为 WebSocketSendBody
    :param bot:
    :param event:
    :return: WebSocketSendBody
    """
    websocket_send_body = WebSocketSendBody()
    websocket_send_body.message = []

    # 发送群聊名称
    if plugin_config.mc_qq_send_group_name:
        group_name = MessageItem()
        group_name.color = TextColor.GREEN
        guild: Guild = await bot.get_guild(guild_id=event.guild_id)
        channel: Channel = await bot.get_channel(channel_id=event.channel_id)
        group_name.msg_text = f"{guild.name}丨{channel.name}"
        websocket_send_body.message.append(group_name)

    member_nickname = await get_qq_member_nickname(bot, event, event.author.id)

    sender_name = MessageItem()
    sender_name.msg_text = member_nickname + " "
    sender_name.color = TextColor.WHITE
    websocket_send_body.message.append(sender_name)

    sender_say = MessageItem()
    sender_say.msg_text = "说："
    sender_say.color = TextColor.GOLD
    websocket_send_body.message.append(sender_say)

    for msg in event.get_message():
        per_msg = MessageItem()
        per_msg.action_event = ActionEvent()

        # 文本
        if msg.type == "text":
            per_msg.msg_text = msg.data['text'].replace("\r", "").replace("\n", "\n * ")
            per_msg.color = TextColor.WHITE
        # 表情
        elif msg.type == "emoji":
            per_msg.msg_text = '[表情]'
            per_msg.color = TextColor.YELLOW
        # @用户
        elif msg.type == "mention_user":
            per_msg.msg_text = f"@{await get_qq_member_nickname(bot, event, msg.data['user_id'])} "
            per_msg.color = TextColor.GREEN
        # @频道
        elif msg.type == "mention_channel":
            per_msg.msg_text = f"@{(await bot.get_channel(channel_id=event.channel_id)).name} "
            per_msg.color = TextColor.GREEN
        # @全体成员
        elif msg.type == "mention_everyone":
            per_msg.msg_text = f"@全体成员 "
            per_msg.color = TextColor.GREEN
        # 图片
        elif msg.type == "image" or "attachment":
            per_msg.color = TextColor.AQUA
            if plugin_config.mc_qq_chat_image_enable:
                img_url = msg.data['url'] if msg.data['url'].startswith("http") else f"http://{msg.data['url']}"
                per_msg.msg_text = str(ChatImageModel(url=img_url, name="图片"))
            else:
                per_msg.msg_text = "[图片]"
                per_msg.action_event.click_event_url = msg.data['url']
                per_msg.action_event.hover_event_text = "[查看图片]"
        # 视频
        else:
            per_msg.msg_text = f"[{msg.type}]"
            per_msg.color = TextColor.WHITE
        websocket_send_body.message.append(per_msg)
    return websocket_send_body


async def parse_onebot_rcon_msg_to_basemodel(
        bot: OneBot,
        event: Union[GroupMessageEvent, GuildMessageEvent]
) -> RconSendBody:
    """
    :param bot: OneBot
    :param event: 事件
    :return: RconSendBody
    """
    rcon_send_body = RconSendBody()

    prefix_component = RconTextComponent()
    prefix_component.text = "[MC_QQ] "
    prefix_component.color = TextColor.YELLOW
    rcon_send_body.message.append(prefix_component)

    if plugin_config.mc_qq_send_group_name:
        group_name_component = RconTextComponent()
        group_name_component.color = TextColor.AQUA

        if isinstance(event, GroupMessageEvent):
            group_name_component.text = f"[{(await bot.get_group_info(group_id=event.group_id))['group_name']}] "
        elif isinstance(event, GuildMessageEvent):
            guild_name = (await bot.get_guild_meta_by_guest(guild_id=event.guild_id))['guild_name']
            for per_channel in (await bot.get_guild_channel_list(guild_id=event.guild_id, no_cache=True)):
                if str(event.channel_id) == per_channel['channel_id']:
                    channel_name = per_channel['channel_name']
                    group_name_component.text = f"[{guild_name}/{channel_name}] "
                    break
        rcon_send_body.message.append(group_name_component)

    member_nickname = await _get_onebot_member_nickname(bot, event, event.user_id) + " "
    sender_component = RconTextComponent()
    sender_component.text = member_nickname
    sender_component.color = TextColor.WHITE
    rcon_send_body.message.append(sender_component)

    sender_say_component = RconTextComponent()
    sender_say_component.text = "说："
    sender_say_component.color = TextColor.YELLOW
    rcon_send_body.message.append(sender_say_component)

    for msg in event.message:
        text_component = RconTextComponent()
        if msg.type == "text":
            text_component.text = msg.data['text'].replace("\r", "").replace("\n", "\n * ")
            text_component.color = TextColor.WHITE
        elif msg.type == "image":
            text_component.text = "[图片]"
            text_component.color = TextColor.AQUA
            if plugin_config.mc_qq_rcon_click_action_enable:
                text_component.clickEvent = RconClickEvent(
                    action=RconClickEventEnum.OPEN_URL,
                    value=msg.data['url']
                )
            if plugin_config.mc_qq_rcon_hover_event_enable:
                text_component.hover_event = RconHoverEvent(
                    action=RconHoverEventEnum.SHOW_TEXT,
                    contents=[RconTextComponent(text="查看图片", color=TextColor.GOLD)]
                )
        elif msg.type == "face":
            text_component.text = '[表情]'
            text_component.color = TextColor.YELLOW
        elif msg.type == "record":
            text_component.text = '[语音]'
            text_component.color = TextColor.GREEN
        elif msg.type == "video":
            text_component.text = '[视频]'
            text_component.color = TextColor.LIGHT_PURPLE
            if plugin_config.mc_qq_rcon_click_action_enable:
                text_component.clickEvent = RconClickEvent(
                    action=RconClickEventEnum.OPEN_URL,
                    value=msg.data['url']
                )
            if plugin_config.mc_qq_rcon_hover_event_enable:
                text_component.hover_event = RconHoverEvent(
                    action=RconHoverEventEnum.SHOW_TEXT,
                    contents=[RconTextComponent(text="查看视频", color=TextColor.DARK_PURPLE)]
                )
        elif msg.type == "at":
            at_member_nickname = await _get_onebot_member_nickname(bot, event, msg.data['qq'])
            text_component.text = f"@{at_member_nickname} "
            text_component.color = TextColor.GREEN
        elif msg.type == "share":
            text_component.text = "[分享]"
            text_component.color = TextColor.YELLOW
        elif msg.type == "forward":
            # TODO 将合并转发消息拼接为字符串
            # 获取合并转发 await bot.get_forward_msg(message_id=event.message_id)
            text_component.text = '[合并转发]'
        else:
            text_component.text = f"[{msg.type}]"
            text_component.color = TextColor.WHITE

        rcon_send_body.message.append(text_component)
    return rcon_send_body


async def parse_qq_rcon_msg_to_basemodel(
        bot: QQBot,
        event: MessageCreateEvent
) -> RconSendBody:
    """
    格式化 QQ频道 消息为 RconSendBody
    :param bot: QQBot
    :param event: 事件
    :return: RconSendBody
    """
    rcon_send_body = RconSendBody()

    prefix_component = RconTextComponent()
    prefix_component.text = "[MC_QQ] "
    prefix_component.color = TextColor.YELLOW
    rcon_send_body.message.append(prefix_component)

    if plugin_config.mc_qq_send_group_name:
        group_name_component = RconTextComponent()
        group_name_component.color = TextColor.AQUA

        guild: Guild = await bot.get_guild(guild_id=event.guild_id)
        channel: Channel = await bot.get_channel(channel_id=event.channel_id)
        group_name_component.text = f"[{guild.name}/{channel.name}] "

        rcon_send_body.message.append(group_name_component)

    member_nickname = await get_qq_member_nickname(bot, event, event.author.id) + " "
    sender_component = RconTextComponent()
    sender_component.text = member_nickname
    sender_component.color = TextColor.WHITE
    rcon_send_body.message.append(sender_component)

    sender_say_component = RconTextComponent()
    sender_say_component.text = "说："
    sender_say_component.color = TextColor.YELLOW
    rcon_send_body.message.append(sender_say_component)

    for msg in event.get_message():
        text_component = RconTextComponent()
        if msg.type == "text":
            text_component.text = msg.data['text'].replace("\r", "").replace("\n", "\n * ")
            text_component.color = TextColor.WHITE
        elif msg.type == "emoji":
            text_component.text = '[表情]'
            text_component.color = TextColor.YELLOW
        elif msg.type == "mention_user":
            text_component.text = f"@{await get_qq_member_nickname(bot, event, msg.data['user_id'])} "
            text_component.color = TextColor.GREEN
        elif msg.type == "mention_channel":
            text_component.text = f"@{(await bot.get_channel(channel_id=event.channel_id)).name} "
            text_component.color = TextColor.GREEN
        elif msg.type == "mention_everyone":
            text_component.text = f"@全体成员 "
            text_component.color = TextColor.GREEN
        elif msg.type == "image":
            text_component.text = "[图片]"
            text_component.color = TextColor.AQUA
            if plugin_config.mc_qq_rcon_click_action_enable:
                text_component.clickEvent = RconClickEvent(
                    action=RconClickEventEnum.OPEN_URL,
                    value=msg.data['url']
                )
            if plugin_config.mc_qq_rcon_hover_event_enable:
                text_component.hover_event = RconHoverEvent(
                    action=RconHoverEventEnum.SHOW_TEXT,
                    contents=[RconTextComponent(text="查看图片", color=TextColor.GOLD)]
                )
        else:
            text_component.text = f"[{msg.type}]"
            text_component.color = TextColor.WHITE
        rcon_send_body.message.append(text_component)
    return rcon_send_body
